import Echo from "laravel-echo";
import PusherPkg from "pusher-js";
import { createConsola } from "consola";
import { useEchoConfig } from "./composables/useEchoConfig.js";
import { useEchoAppConfig } from "./composables/useEchoAppConfig.js";
import { createError, defineNuxtPlugin, useCookie, updateAppConfig } from "#app";
const Pusher = PusherPkg.default || PusherPkg;
const MODULE_NAME = "nuxt-laravel-echo";
function createEchoLogger(logLevel) {
  return createConsola({ level: logLevel }).withTag(MODULE_NAME);
}
const readCsrfCookie = (name) => useCookie(name, { readonly: true });
function createFetchClient(app, authentication, logger) {
  const fetchOptions = {
    baseURL: authentication.baseUrl,
    credentials: "include",
    retry: false,
    async onRequest(context) {
      if (authentication.mode === "cookie") {
        let csrfToken = readCsrfCookie(authentication.csrfCookie);
        if (!csrfToken.value) {
          await $fetch(authentication.csrfEndpoint, {
            baseURL: authentication.baseUrl,
            credentials: "include",
            retry: false
          });
          csrfToken = readCsrfCookie(authentication.csrfCookie);
        }
        if (!csrfToken.value) {
          logger.warn(`${authentication.csrfCookie} cookie is missing, unable to set ${authentication.csrfHeader} header`);
          return;
        }
        context.options.headers.set(authentication.csrfHeader, csrfToken.value);
      }
      if (authentication.mode === "token") {
        const { tokenStorage } = useEchoAppConfig();
        if (!tokenStorage) {
          throw createError("Token storage is not defined");
        }
        const token = await tokenStorage.get(app);
        if (!token) {
          logger.debug("Authorization token is missing, unable to set header");
          return;
        }
        context.options.headers.set("Authorization", `Bearer ${token}`);
      }
    }
  };
  return $fetch.create(fetchOptions);
}
function createAuthorizer(app, authentication, logger) {
  const client = createFetchClient(app, authentication, logger);
  return (channel, _) => {
    return {
      authorize: (socketId, callback) => {
        const payload = JSON.stringify({
          socket_id: socketId,
          channel_name: channel.name
        });
        client(authentication.authEndpoint, {
          method: "post",
          body: payload
        }).then(
          (response) => callback(null, response)
        ).catch((error) => callback(error, null));
      }
    };
  };
}
function prepareEchoOptions(app, config, logger) {
  const forceTLS = config.scheme === "https";
  const additionalOptions = config.properties || {};
  const authorizer = config.authentication ? createAuthorizer(
    app,
    config.authentication,
    logger
  ) : void 0;
  if (config.broadcaster === "pusher") {
    if (!forceTLS) {
      throw createError('Pusher requires a secure connection (schema: "https")');
    }
    return {
      broadcaster: config.broadcaster,
      key: config.key,
      cluster: config.cluster,
      forceTLS,
      authorizer,
      ...additionalOptions
    };
  }
  return {
    broadcaster: config.broadcaster,
    key: config.key,
    wsHost: config.host,
    wsPort: config.port,
    wssPort: config.port,
    forceTLS,
    enabledTransports: config.transports,
    authorizer,
    ...additionalOptions
  };
}
async function setupDefaultTokenStorage(nuxtApp, logger) {
  logger.debug(
    "Token storage is not defined, switch to default cookie storage"
  );
  const defaultStorage = await import("./storages/cookieTokenStorage.js");
  nuxtApp.runWithContext(() => {
    updateAppConfig({
      echo: {
        tokenStorage: defaultStorage.cookieTokenStorage
      }
    });
  });
}
export default defineNuxtPlugin(async (_nuxtApp) => {
  const nuxtApp = _nuxtApp;
  const config = useEchoConfig();
  const appConfig = useEchoAppConfig();
  const logger = createEchoLogger(config.logLevel);
  if (config.authentication?.mode === "token" && !appConfig.tokenStorage) {
    await setupDefaultTokenStorage(nuxtApp, logger);
  }
  window.Pusher = Pusher;
  window.Echo = new Echo(prepareEchoOptions(nuxtApp, config, logger));
  logger.debug("Laravel Echo client initialized");
  return {
    provide: {
      echo: window.Echo
    }
  };
});
